@@ -124,8 +124,7 @@ class GifImageFile(ImageFile.ImageFile):
         if not self._seek_check(frame):
             return
         if frame < self.__frame:
-            if frame != 0:
-                self.im = None
+            self.im = None
             self._seek(0)
 
         last_frame = self.__frame
@@ -165,12 +164,21 @@ class GifImageFile(ImageFile.ImageFile):
                 pass
             self.__offset = 0
 
+        if self.__frame == 1:
+            self.pyaccess = None
+            if "transparency" in self.info:
+                self.mode = "RGBA"
+                self.im.putpalettealpha(self.info["transparency"], 0)
+                self.im = self.im.convert("RGBA", Image.FLOYDSTEINBERG)
+
+                del self.info["transparency"]
+            else:
+                self.mode = "RGB"
+                self.im = self.im.convert("RGB", Image.FLOYDSTEINBERG)
         if self.dispose:
             self.im.paste(self.dispose, self.dispose_extent)
 
-        from copy import copy
-
-        self.palette = copy(self.global_palette)
+        palette = None
 
         info = {}
         frame_transparency = None
@@ -246,7 +254,7 @@ class GifImageFile(ImageFile.ImageFile):
 
                 if flags & 128:
                     bits = (flags & 7) + 1
-                    self.palette = ImagePalette.raw("RGB", self.fp.read(3 << bits))
+                    palette = ImagePalette.raw("RGB", self.fp.read(3 << bits))
 
                 # image data
                 bits = self.fp.read(1)[0]
@@ -257,6 +265,15 @@ class GifImageFile(ImageFile.ImageFile):
                 pass
                 # raise OSError, "illegal GIF tag `%x`" % s[0]
 
+        frame_palette = palette or self.global_palette
+
+        def _rgb(color):
+            if frame_palette:
+                color = tuple(frame_palette.palette[color * 3 : color * 3 + 3])
+            else:
+                color = (color, color, color)
+            return color
+
         try:
             if self.disposal_method < 2:
                 # do not dispose or none specified
@@ -271,12 +288,14 @@ class GifImageFile(ImageFile.ImageFile):
                 Image._decompression_bomb_check(dispose_size)
 
                 # by convention, attempt to use transparency first
-                color = (
-                    frame_transparency
-                    if frame_transparency is not None
-                    else self.info.get("background", 0)
-                )
-                self.dispose = Image.core.fill("P", dispose_size, color)
+                color = self.info.get("transparency", frame_transparency)
+                if color is not None:
+                    dispose_mode = "RGBA"
+                    color = _rgb(color) + (0,)
+                else:
+                    dispose_mode = "RGB"
+                    color = _rgb(self.info.get("background", 0))
+                self.dispose = Image.core.fill(dispose_mode, dispose_size, color)
             else:
                 # replace with previous contents
                 if self.im:
@@ -288,24 +307,20 @@ class GifImageFile(ImageFile.ImageFile):
 
                     Image._decompression_bomb_check(dispose_size)
                     self.dispose = Image.core.fill(
-                        "P", dispose_size, frame_transparency
+                        "RGBA", dispose_size, _rgb(frame_transparency) + (0,)
                     )
         except AttributeError:
             pass
 
         if interlace is not None:
-            transparency = -1
-            if frame_transparency is not None:
-                if frame == 0:
-                    self.info["transparency"] = frame_transparency
-                else:
-                    transparency = frame_transparency
+            if frame == 0 and frame_transparency is not None:
+                self.info["transparency"] = frame_transparency
             self.tile = [
                 (
                     "gif",
                     (x0, y0, x1, y1),
                     self.__offset,
-                    (bits, interlace, transparency),
+                    (bits, interlace),
                 )
             ]
         else:
@@ -318,15 +333,53 @@ class GifImageFile(ImageFile.ImageFile):
             elif k in self.info:
                 del self.info[k]
 
-        self.mode = "L"
-        if self.palette:
-            self.mode = "P"
+        if frame == 0:
+            self.mode = "P" if frame_palette else "L"
+
+            if self.mode == "P" and not palette:
+                from copy import copy
+
+                palette = copy(self.global_palette)
+            self.palette = palette
+        else:
+            self._frame_palette = frame_palette
+            self._frame_transparency = frame_transparency
 
     def load_prepare(self):
-        if not self.im and "transparency" in self.info:
-            self.im = Image.core.fill(self.mode, self.size, self.info["transparency"])
+        if self.__frame == 0:
+            if "transparency" in self.info:
+                self.im = Image.core.fill(
+                    self.mode, self.size, self.info["transparency"]
+                )
+        else:
+            self._prev_im = self.im
+            if self._frame_palette:
+                self.mode = "P"
+                self.im = Image.core.fill("P", self.size, self._frame_transparency or 0)
+                self.im.putpalette(*self._frame_palette.getdata())
+                self._frame_palette = None
+            else:
+                self.mode = "L"
+                self.im = None
 
-        super(GifImageFile, self).load_prepare()
+        super().load_prepare()
+
+    def load_end(self):
+        if self.__frame == 0:
+            return
+        if self._frame_transparency is not None:
+            self.im.putpalettealpha(self._frame_transparency, 0)
+            frame_im = self.im.convert("RGBA")
+        else:
+            frame_im = self.im.convert("RGB")
+        frame_im = self._crop(frame_im, self.dispose_extent)
+
+        self.im = self._prev_im
+        self.mode = self.im.mode
+        if frame_im.mode == "RGBA":
+            self.im.paste(frame_im, self.dispose_extent, frame_im)
+        else:
+            self.im.paste(frame_im, self.dispose_extent)
 
     def tell(self):
         return self.__frame
@@ -372,7 +425,13 @@ def _normalize_mode(im, initial_call=False):
             palette_size = 256
             if im.palette:
                 palette_size = len(im.palette.getdata()[1]) // 3
-            return im.convert("P", palette=Image.ADAPTIVE, colors=palette_size)
+            im = im.convert("P", palette=Image.ADAPTIVE, colors=palette_size)
+            if im.palette.mode == "RGBA":
+                for rgba in im.palette.colors.keys():
+                    if rgba[3] == 0:
+                        im.info["transparency"] = im.palette.colors[rgba]
+                        break
+            return im
         else:
             return im.convert("P")
     return im.convert("L")
@@ -406,9 +465,26 @@ def _normalize_palette(im, palette, info):
             source_palette = bytearray(i // 3 for i in range(768))
         im.palette = ImagePalette.ImagePalette("RGB", palette=source_palette)
 
-    used_palette_colors = _get_optimize(im, info)
-    if used_palette_colors is not None:
-        return im.remap_palette(used_palette_colors, source_palette)
+    if palette:
+        used_palette_colors = []
+        for i in range(0, len(source_palette), 3):
+            source_color = tuple(source_palette[i : i + 3])
+            try:
+                index = im.palette.colors[source_color]
+            except KeyError:
+                index = None
+            used_palette_colors.append(index)
+        for i, index in enumerate(used_palette_colors):
+            if index is None:
+                for j in range(len(used_palette_colors)):
+                    if j not in used_palette_colors:
+                        used_palette_colors[i] = j
+                        break
+        im = im.remap_palette(used_palette_colors)
+    else:
+        used_palette_colors = _get_optimize(im, info)
+        if used_palette_colors is not None:
+            return im.remap_palette(used_palette_colors, source_palette)
 
     im.palette.palette = source_palette
     return im
@@ -499,7 +575,8 @@ def _write_multiple_frames(im, fp, palette):
                 offset = (0, 0)
             else:
                 # compress difference
-                frame_data["encoderinfo"]["include_color_table"] = True
+                if not palette:
+                    frame_data["encoderinfo"]["include_color_table"] = True
 
                 im_frame = im_frame.crop(frame_data["bbox"])
                 offset = frame_data["bbox"][:2]
@@ -779,7 +856,7 @@ def _get_global_header(im, info):
     """Return a list of strings representing a GIF header"""
 
     # Header Block
-    # http://www.matthewflickinger.com/lab/whatsinagif/bits_and_bytes.asp
+    # https://www.matthewflickinger.com/lab/whatsinagif/bits_and_bytes.asp
 
     version = b"87a"
     for extensionKey in ["transparency", "duration", "loop", "comment"]:
